from future import standard_library
standard_library.install_aliases()
from collections import Mapping, namedtuple, UserDict


def mapper(mapping, _nt_name='NT'):
    """ Convert mappings to namedtuples recursively. """
    if isinstance(mapping, Mapping) and not isinstance(mapping, AsDict):
        for key, value in list(mapping.items()):
            mapping[key] = mapper(value)
        return namedtuple_wrapper(_nt_name, **mapping)
    elif isinstance(mapping, list):
        return [mapper(item) for item in mapping]
    return mapping


def namedtuple_wrapper(_nt_name, **kwargs):
    wrap = namedtuple(_nt_name, kwargs)
    return wrap(**kwargs)


class AsDict(UserDict):
    """ A class that exists just to tell `mapper` not to eat it. """


def ignore(mapping):
    """ Use ignore to prevent a mapping from being mapped to a namedtuple. """
    if isinstance(mapping, Mapping):
        return AsDict(mapping)
    elif isinstance(mapping, list):
        return [ignore(item) for item in mapping]
    return mapping


def isnamedtupleinstance(x):  # thank you http://stackoverflow.com/a/2166841/6085135
    _type = type(x)
    bases = _type.__bases__
    if len(bases) != 1 or bases[0] != tuple:
        return False
    fields = getattr(_type, '_fields', None)
    if not isinstance(fields, tuple):
        return False
    return all(type(i)==str for i in fields)


def reducer(obj):
    if isinstance(obj, dict):
        return {key: reducer(value) for key, value in obj.items()}
    elif isinstance(obj, list):
        return [reducer(value) for value in obj]
    elif isnamedtupleinstance(obj):
        return {key: reducer(value) for key, value in obj._asdict().items()}
    elif isinstance(obj, tuple):
        return tuple(reducer(value) for value in obj)
    else:
        return obj



from .namedtupled import mapper, namedtuple
import yaml
import json
import os
import getpass


def load_lists(keys=[], values=[], name='NT'):
    """ Map namedtuples given a pair of key, value lists. """
    mapping = dict(zip(keys, values))
    return mapper(mapping, _nt_name=name)


def load_json(data=None, path=None, name='NT'):
    """ Map namedtuples with json data. """
    if data and not path:
        return mapper(json.loads(data), _nt_name=name)
    if path and not data:
        return mapper(json.load(path), _nt_name=name)
    if data and path:
        raise ValueError('expected one source and received two')


def load_yaml(data=None, path=None, name='NT'):
    """ Map namedtuples with yaml data. """
    if data and not path:
        return mapper(yaml.load(data), _nt_name=name)
    if path and not data:
        with open(path, 'r') as f:
            data = yaml.load(f)
        return mapper(data, _nt_name=name)
    if data and path:
        raise ValueError('expected one source and received two')


def load_env(keys=[], name='NT', use_getpass=False):
    """ Returns a namedtuple from a list of environment variables.
    If not found in shell, gets input with *input* or *getpass*. """
    NT = namedtuple(name, keys)
    if use_getpass:
        values = [os.getenv(x) or getpass.getpass(x) for x in keys]
    else:
        values = [os.getenv(x) or input(x) for x in keys]
    return NT(*values)



from setuptools import setup


brief_description = """
.. image:: https://travis-ci.org/brennv/namedtupled.svg?branch=master
    :target: https://travis-ci.org/brennv/namedtupled
.. image:: https://img.shields.io/badge/python-2.7%2C%203.4%2C%203.5-blue.svg
.. image:: https://img.shields.io/codecov/c/github/brennv/namedtupled.svg
    :target: https://codecov.io/gh/brennv/namedtupled

Source: `https://github.com/brennv/namedtupled`_
Docs: `http://namedtupled.readthedocs.io/en/latest/`_

`namedtuples`_ are immutable, performant and classy. **namedtupled** is
a lightweight wrapper for recursively creating namedtuples from nested
dicts, lists, json and yaml. Thanks to `hangtwenty`_.

Installation
============

.. code:: bash

    pip install namedtupled

Getting started
===============

.. code:: python

    import namedtupled

    data = {'binks': {'says': 'meow'}}
    cat = namedtupled.map(data)

    cat  # NT(binks=NT(says='meow'))

    cat.binks.says  # 'meow'

.. _namedtuples: https://docs.python.org/3/library/collections.html
.. _hangtwenty: https://gist.github.com/hangtwenty/5960435
.. _https://github.com/brennv/namedtupled: https://github.com/brennv/namedtupled
.. _http://namedtupled.readthedocs.io/en/latest/: http://namedtupled.readthedocs.io/en/latest/
"""


setup(
    name='namedtupled',
    packages=['namedtupled'],
    version='0.3.1',
    description='Lightweight wrapper for creating namedtuples.',
    long_description=brief_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    author='brennv',
    author_email='brennan@beta.build',
    license='MIT',
    url='https://github.com/brennv/namedtupled',
    download_url='https://github.com/brennv/namedtupled/tarball/0.3.1',
    keywords='namedtupled namedtuple json yaml',
    install_requires=[
        'future',
        'pyyaml',
    ],
    include_package_data=True,
    zip_safe=False)



from .namedtupled import mapper, ignore, reducer
from .integrations import load_json, load_yaml, load_lists, load_env


map = mapper
reduce = reducer
json = load_json
yaml = load_yaml
zip = load_lists
env = load_env
